function [labels, shadowLabels, bf1param, bf2param, recon, err] = NumBlurLinShadowOpt(img, iteNum, kernelSize, inIdx, inShadowIdx, in1param, in2param)

    GlobalSetting;
    
    if kernelSize>1
        kernel = fspecial('gaussian', [kernelSize kernelSize], kernelSize/4);
    else
        kernel = 1;
    end
    %kernel = 1;

    [h, w, clr]= size(img);

    sizeScale = (w^2+h^2)^0.5;
    idx = 1:(w*h);
    idx = idx';
    [x, y] = ind2sub([h, w], idx);
    pixPst = [x,y]/sizeScale;
    pixClr = [img(idx), img(idx+w*h), img(idx+w*h*2)];

    labels=zeros(h*w, 1);
    shadowLabels = zeros(h*w, 1);
        
    if (nargin>4)
        
        bf1param = in1param;
        bf2param = in2param;
        
        idx1 = find(inIdx==1);
        idx2 = find(inIdx==2);        
        
        shadowLabels = inShadowIdx;
        shadowLabels = reshape(shadowLabels, [h*w, 1]);
        
    else

        bf1param = EncodeParam(rand()*10, rand()*pi, rand()*2*pi, rand(1,3), rand()*pi, rand()*2*pi, rand()*100, rand(1,3)*255, rand(1,3)*255, rand(1,3)*1, rand(1,1));
        %bf2param = EncodeParam(rand()*10, rand()*pi, rand()*2*pi, rand()*pi, rand()*2*pi, rand()*100, rand(1,3)*255, rand(1,3)*255);
        bf2param = bf1param;

        clusteridx = kmeans(rgb2hsv(pixClr), 2);

        idx1 = find(clusteridx==1);
        idx2 = find(clusteridx==2);
        
        grayimg = 0.2989 * pixClr(:, 1) + 0.5870 * pixClr(:, 2) + 0.1140 * pixClr(:, 3);
        
        for i=1:2
            idxlist = find(clusteridx==i);

            graynow = grayimg(idxlist);

            [shadowclusteridx, c] = kmeans(graynow, 2);
            if c(1)<c(2)
                shadowList = idxlist(shadowclusteridx==1);
            else
                shadowList = idxlist(shadowclusteridx==2);
            end
            
            shadowLabels(shadowList) = 1;
        end
    end

    lightDrt = (bf1param(1, 4:6)+bf2param(1,4:6))/2;
    malpha = [bf1param(1, 1), bf2param(1, 1)];
    
    pixClr1 = pixClr(idx1,:);
    pixPst1 = pixPst(idx1,:);
    pixClr2 = pixClr(idx2,:);
    pixPst2 = pixPst(idx2,:);
    labels(idx1) = 1;
    labels(idx2) = 2;
    
    labels1 = ones(h*w, 1);
    labels2 = ones(h*w, 1)*2;
    
    shadowAll = ones(h*w, 1);
    shadowNo = zeros(h*w, 1);
    
    predImg = zeros(h, w, 3);

    for t=1:iteNum
        if useSpec==1
            tmpResult = fsolve(@(param)BlurOptShadow(param(1:3)', param(4:5)', kernel, labels, shadowLabels, pixPst, pixClr, img), [lightDrt, malpha]');
            lightDrt = tmpResult(1:3)';
            malpha = tmpResult(4:5)';
        else
            lightDrt = fsolve(@(param)BlurOptShadow(param, [0,0]', kernel, labels, shadowLabels, pixPst, pixClr, img), lightDrt')';
        end
        [Cd, Cs, Ca] = CalcBlurShadow(pixPst, pixClr, img, labels, shadowLabels, kernel, lightDrt, malpha);
        recon1 = BPShadow(lightDrt, malpha, Cd, Cs, Ca, labels1, shadowNo, pixPst, img);
        recon1Shadow = BPShadow(lightDrt, malpha, Cd, Cs, Ca, labels1, shadowAll, pixPst, img);
        recon2 = BPShadow(lightDrt, malpha, Cd, Cs, Ca, labels2, shadowNo, pixPst, img);
        recon2Shadow = BPShadow(lightDrt, malpha, Cd, Cs, Ca, labels2, shadowAll, pixPst, img);
        [newLabels, shadowLabels, recon, err] = DeblurEmbedShadow(recon1, recon1Shadow, recon2, recon2Shadow, img, kernel);
        
        %figure(1);
        %imshow(newLabels-1);
        %figure(2);
        %imshow(recon/255);
        %figure(3);
        %imshow(shadowLabels);
        %drawnow;

        labels = reshape(newLabels, [h*w, 1]);
        shadowLabels = reshape(shadowLabels, [h*w, 1]);
    end
    labels = reshape(labels, [h, w]);
    shadowLabels = reshape(shadowLabels, [h, w]);
    bf1param(1, 13:15) = Cd(1, :);
    bf2param(1, 13:15) = Cd(2, :);
    bf1param(1, 16:18) = Cs(1, :);
    bf2param(1, 16:18) = Cs(2, :);
    bf1param(1, 1) = malpha(1);
    bf2param(1, 1) = malpha(2);
    bf1param(1, 4:6) = lightDrt;
    bf2param(1, 4:6) = lightDrt;
    